package data.scripts.wyv;

import java.awt.Color;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.econ.CommoditySpecAPI;
import com.fs.starfarer.api.impl.campaign.ids.Stats;
import com.fs.starfarer.api.impl.campaign.ids.Strings;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.impl.campaign.econ.impl.BaseIndustry;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.population.PopulationComposition;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;


public class PlanetaryShield extends BaseIndustry {

	private static final String BASE_DESC = "An envelope of charged plasma seeded with exotic virtual particles producing effects which disrupt the combat effectiveness of long-range bombardment and even incursion by small vessels. The planetary shield can be continually charged and controlled by a central source and presents a formidable obstacle to any form of ground assault.";
	private static final String MOD_DESC = BASE_DESC + "\n\nIt also presents a formidable obstacle to any unscheduled transfer of goods or personnel, reducing the odds of Pather terrorists gaining a foothold on the ground... If the colony actually has customs inspectors in the first place, of course.";
	private static final String METEOR_DESC = MOD_DESC + "\n\nAnd, last but not least, it offers almost-perfect protection against the planet's frequent meteor impacts - though there are still significant maintenance costs incurred to ensure that a shield failure would not prove to be a colony-ending disaster.";
	
	public void setNewBuild( float progress ) {
		building = true;
		buildProgress = spec.getBuildTime() * progress;
		upgradeId = null;
		buildTime = spec.getBuildTime();
	}
	
	public void apply() {
		super.apply(false);
		
		int size = 5;
		applyIncomeAndUpkeep(size);
		
		if( PSModPlugin.revertToVanilla ) {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId() );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyMult( getModId(), 3f, getNameForModifier() );
			market.getHazard().unmodifyFlat(getModId());
		} else {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId() );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyFlat( getModId(), 100f, getNameForModifier() );
			if( null != market.getFirstCondition( "meteor_impacts" ) ) {
				market.getHazard().modifyFlat(getModId(), -0.25f, getNameForModifier());
			} else {
				market.getHazard().unmodifyFlat( getModId() );
			}
		}
		
		if (isFunctional()) {
			applyVisuals(market.getPlanetEntity());
		} else {
			unapply();
		}
	}
	
	public static void applyVisuals(PlanetAPI planet) {
		if (planet == null) return;
		planet.getSpec().setShieldTexture(Global.getSettings().getSpriteName("industry", "shield_texture"));
		planet.getSpec().setShieldThickness(0.1f);
		planet.getSpec().setShieldColor(new Color(255,255,255,175));
		planet.applySpecChanges();
	}
	
	public static void unapplyVisuals(PlanetAPI planet) {
		if (planet == null) return;
		planet.getSpec().setShieldTexture(null);
		planet.getSpec().setShieldThickness(0f);
		planet.getSpec().setShieldColor(null);
		planet.applySpecChanges();
	}

	
	@Override
	public void unapply() {
		super.unapply();
		
		market.getHazard().unmodifyFlat( getModId() );
		
		unapplyVisuals(market.getPlanetEntity());
		
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult(getModId());
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat(getModId());
	}

	@Override
	public boolean isAvailableToBuild() {
		if (!Global.getSector().getPlayerFaction().knowsIndustry(getId())) {
			return false;
		}
		return market.getPlanetEntity() != null && !market.getPlanetEntity().isGasGiant();
	}
	
	@Override
	public String getUnavailableReason() {
		if (!super.isAvailableToBuild()) return super.getUnavailableReason();
		return "Can not be built at a gas giant";
	}
	
	public boolean showWhenUnavailable() {
		return Global.getSector().getPlayerFaction().knowsIndustry(getId());
	}

	protected boolean hasPostDemandSection(boolean hasDemand, IndustryTooltipMode mode) {
		return mode != IndustryTooltipMode.NORMAL || isFunctional();
	}
	
	@Override
	protected void addPostDemandSection(TooltipMakerAPI tooltip, boolean hasDemand, IndustryTooltipMode mode) {
		if (mode != IndustryTooltipMode.NORMAL || isFunctional()) {
			Color h = Misc.getHighlightColor();
			float opad = 10f;
			
			if( PSModPlugin.revertToVanilla ) {
				tooltip.addPara( "Ground defense strength: %s", opad, h, "x3" );
			} else {
				tooltip.addPara( "Ground defense strength: %s", opad, h, "+100" );
				if( null != market.getFirstCondition( "meteor_impacts" ) ) {
					tooltip.addPara( "Hazard rating: %s", opad, h, "-25%" );
				}
				if( !getMarket().isFreePort() ) {
					tooltip.addPara( "Pather interest: %s", opad, h, "-10" );
				}
			}
		}
	}
	
	
	@Override
	protected void applyAlphaCoreModifiers() {
		if( PSModPlugin.revertToVanilla ) {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyMult( getModId(1), 1.5f, "Alpha core (" + getNameForModifier() + ")");
		} else {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyFlat( getModId(1), 50f, "Alpha core (" + getNameForModifier() + ")");
		}
	}
	
	//@Override
	//protected void applyBetaCoreModifiers() {
	//	market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
	//	market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
	//}
	
	//@Override
	//protected void applyGammaCoreModifiers() {
	//	market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
	//	market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
	//}
	
	@Override
	protected void applyNoAICoreModifiers() {
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat( getModId(1) );
		market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult( getModId(1) );
	}
	
	@Override
	protected void applyAlphaCoreSupplyAndDemandModifiers() {
		demandReduction.modifyFlat(getModId(0), DEMAND_REDUCTION, "Alpha core");
	}
	
	@Override
	protected void addAlphaCoreDescription(TooltipMakerAPI tooltip, AICoreDescriptionMode mode) {
		float opad = 10f;
		Color highlight = Misc.getHighlightColor();
		
		String pre = "Alpha-level AI core currently assigned. ";
		if (mode == AICoreDescriptionMode.MANAGE_CORE_DIALOG_LIST || mode == AICoreDescriptionMode.INDUSTRY_TOOLTIP) {
			pre = "Alpha-level AI core. ";
		}
		String modifierString;
		if( PSModPlugin.revertToVanilla ) {
			modifierString = "x1.5";
		} else {
			modifierString = "+50";
		}
		String patherInterestString = "";
		if( !PSModPlugin.revertToVanilla && !getMarket().isFreePort() ) {
			patherInterestString = " Reduces Pather interest by %s.";
		}
		
		if (mode == AICoreDescriptionMode.INDUSTRY_TOOLTIP) {
			CommoditySpecAPI coreSpec = Global.getSettings().getCommoditySpec(aiCoreId);
			TooltipMakerAPI text = tooltip.beginImageWithText(coreSpec.getIconName(), 48);
			text.addPara(pre + "Reduces upkeep cost by %s. Reduces demand by %s unit. " +
					"Increases ground defenses by %s." + patherInterestString, 0f, highlight,
					"" + (int)((1f - UPKEEP_MULT) * 100f) + "%", "" + DEMAND_REDUCTION,
					modifierString, "-30" );
			tooltip.addImageWithText(opad);
			return;
		}
		
		tooltip.addPara(pre + "Reduces upkeep cost by %s. Reduces demand by %s unit. " +
				"Increases ground defenses by %s." + patherInterestString, opad, highlight,
				"" + (int)((1f - UPKEEP_MULT) * 100f) + "%", "" + DEMAND_REDUCTION,
				modifierString, "-30" );
		
	}
	
	@Override
	protected String getDescriptionOverride() {
		if( PSModPlugin.revertToVanilla ) {
			return BASE_DESC;
		} else if( null != market.getFirstCondition( "meteor_impacts" ) ) {
			return METEOR_DESC;
		} else {
			return MOD_DESC;
		}
	}

	@Override
	public float getPatherInterest() {
		float interest = super.getPatherInterest();
		if( PSModPlugin.revertToVanilla ) {
			return interest;
		}
		if( !isFunctional() || getMarket().isFreePort() ) return interest;
		interest = interest - 10f;
		if( isImproved() ) {
			interest = interest -10f;
		}
		if( Commodities.ALPHA_CORE.equals( aiCoreId ) ) {
			interest = interest -30f;
		}
		return interest;
	}

	
	@Override
	public boolean canImprove() {
		return true;
	}
	
	protected void applyImproveModifiers() {
		if( PSModPlugin.revertToVanilla ) {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat(getModId(2));
			if (isImproved()) {
				market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyMult(getModId(2),
							1.25f,
							getImprovementsDescForModifiers() + " (" + getNameForModifier() + ")");
			} else {
				market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult(getModId(2));
			}
		} else {
			market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyMult(getModId(2));
			if (isImproved()) {
				market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).modifyFlat(getModId(2),
							50,
							getImprovementsDescForModifiers() + " (" + getNameForModifier() + ")");
			} else {
				market.getStats().getDynamic().getMod(Stats.GROUND_DEFENSES_MOD).unmodifyFlat(getModId(2));
			}
		}
	}
	
	public void addImproveDesc(TooltipMakerAPI info, ImprovementDescriptionMode mode) {
		float opad = 10f;
		Color highlight = Misc.getHighlightColor();
		
		String str;
		if( PSModPlugin.revertToVanilla ) {
			str = Strings.X + (1.25f) + "";
		} else {
			str = "+50";
		}
		
		String patherInterestString = "";
		if( !PSModPlugin.revertToVanilla && !getMarket().isFreePort() ) {
			patherInterestString = " Reduces Pather interest by %s.";
		}
		
		if (mode == ImprovementDescriptionMode.INDUSTRY_TOOLTIP) {
			info.addPara("Ground defenses increased by %s." + patherInterestString, 0f, highlight, str, "-10");
		} else {
			info.addPara("Increases ground defenses by %s." + patherInterestString, 0f, highlight, str, "-10");
		}

		info.addSpacer(opad);
		super.addImproveDesc(info, mode);
	}
	
}
